@@ -52,6 +52,7 @@
 #include <KJobWidgets>
 #include <KIO/StatJob>
 #include <KMessageBox>
+#include <KParts/OpenUrlArguments>
 #include <KPluginFactory>
 #include <KRun>
 #include <KSelectAction>
@@ -59,6 +60,7 @@
 #include <KToggleAction>
 #include <KLocalizedString>
 #include <KXMLGUIFactory>
+#include <kparts_version.h>
 
 #include <QAction>
 #include <QCursor>
@@ -81,8 +83,6 @@
 
 using namespace Kerfuffle;
 
-K_PLUGIN_FACTORY_WITH_JSON(Factory, "ark_part.json", registerPlugin<Ark::Part>();)
-
 namespace Ark
 {
 
@@ -213,6 +213,8 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
             this, &Part::setBusyGui);
     connect(this, &Part::ready,
             this, &Part::setReadyGui);
+    connect(this, &KParts::ReadOnlyPart::urlChanged,
+            this, &Part::setFileNameFromArchive);
     connect(this, static_cast<void (KParts::ReadOnlyPart::*)()>(&KParts::ReadOnlyPart::completed),
             this, &Part::setFileNameFromArchive);
     connect(ArkSettings::self(), &KCoreConfigSkeleton::configChanged, this, &Part::updateActions);
@@ -408,7 +410,7 @@ void Part::setupActions()
     m_renameFileAction->setText(i18n("&Rename"));
     actionCollection()->setDefaultShortcut(m_renameFileAction, Qt::Key_F2);
     m_renameFileAction->setToolTip(i18nc("@info:tooltip", "Click to rename the selected file"));
-    connect(m_renameFileAction, &QAction::triggered, this, &Part::slotEditFileName);
+    connect(m_renameFileAction, &QAction::triggered, m_view, &ArchiveView::renameSelectedEntry);
 
     m_deleteFilesAction = actionCollection()->addAction(QStringLiteral("delete"));
     m_deleteFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-remove")));
@@ -475,7 +477,7 @@ void Part::setupActions()
 
 void Part::updateActions()
 {
-    bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly();
+    const bool isWritable = isArchiveWritable();
     const Archive::Entry *entry = m_model->entryForIndex(m_filterModel->mapToSource(m_view->selectionModel()->currentIndex()));
     int selectedEntriesCount = m_view->selectionModel()->selectedRows().count();
 
@@ -607,6 +609,11 @@ void Part::slotTestArchive()
     job->start();
 }
 
+bool Part::isArchiveWritable() const
+{
+    return isReadWrite() && m_model->archive() && !m_model->archive()->isReadOnly();
+}
+
 void Part::createArchive()
 {
     const QString fixedMimeType = arguments().metaData()[QStringLiteral("fixedMimeType")];
@@ -772,11 +779,12 @@ bool Part::openFile()
 
     if (creatingNewArchive) {
         createArchive();
-    } else {
-        loadArchive();
+        return true;
     }
 
-    return true;
+    loadArchive();
+    // Loading is async, we don't know yet whether we got a valid archive.
+    return false;
 }
 
 bool Part::saveFile()
@@ -886,7 +894,15 @@ void Part::slotLoadingFinished(KJob *job)
             m_infoPanel->updateWithDefaults();
 
             emit setWindowCaption(QString());
+#if KPARTS_VERSION >= QT_VERSION_CHECK(5, 37, 0)
+            // See https://phabricator.kde.org/D6856
+            closeUrl();
+#else
+            setUrl(QUrl());
+#endif
         }
+    } else {
+        emit completed();
     }
 
     m_view->sortByColumn(0, Qt::AscendingOrder);
@@ -894,8 +910,8 @@ void Part::slotLoadingFinished(KJob *job)
 
     // After loading all files, resize the columns to fit all fields
     m_view->header()->resizeSections(QHeaderView::ResizeToContents);
-    // Now we can start accepting drops in the archive view (if loading was successful).
-    m_view->setDropsEnabled(!job->error());
+    // Now we can start accepting drops in the archive view (if loading was successful and the archive can be modified).
+    m_view->setDropsEnabled(!job->error() && isArchiveWritable());
 
     updateActions();
 
@@ -1016,19 +1032,14 @@ void Part::slotOpenExtractedEntry(KJob *job)
         m_tmpExtractDirList << openJob->tempDir();
 
         const QString fullName = openJob->validatedFilePath();
-
-        bool isWritable = m_model->archive() && !m_model->archive()->isReadOnly();
-
-        // If archive is readonly set temporarily extracted file to readonly as
-        // well so user will be notified if trying to modify and save the file.
-        if (!isWritable) {
-            QFile::setPermissions(fullName, QFileDevice::ReadOwner | QFileDevice::ReadGroup | QFileDevice::ReadOther);
-        }
-
-        if (isWritable) {
+        if (isArchiveWritable()) {
             m_fileWatcher = new QFileSystemWatcher;
             connect(m_fileWatcher, &QFileSystemWatcher::fileChanged, this, &Part::slotWatchedFileModified);
             m_fileWatcher->addPath(fullName);
+        } else {
+            // If archive is readonly set temporarily extracted file to readonly as
+            // well so user will be notified if trying to modify and save the file.
+            QFile::setPermissions(fullName, QFileDevice::ReadOwner | QFileDevice::ReadGroup | QFileDevice::ReadOther);
         }
 
         if (qobject_cast<OpenWithJob*>(job)) {
@@ -1415,15 +1426,6 @@ void Part::slotAddFiles()
     delete dlg;
 }
 
-void Part::slotEditFileName()
-{
-    QModelIndex currentIndex = m_view->selectionModel()->currentIndex();
-    currentIndex = (currentIndex.parent().isValid())
-                   ? currentIndex.parent().child(currentIndex.row(), 0)
-                   : m_filterModel->index(currentIndex.row(), 0);
-    m_view->openEntryEditor(currentIndex);
-}
-
 void Part::slotCutFiles()
 {
     QModelIndexList selectedRows = addChildren(getSelectedIndexes());
